
// The class that runs in the slave JVM.

import java.io.* ;
import java.net.* ;
import java.rmi.* ;
import java.rmi.registry.* ;
import java.rmi.server.UnicastRemoteObject ;

public class Slave {

    // First argument is name of the class of the instantiated object.

    // Second argument is the port of the RMI registry holding the
    //        call-back channel.

    public static void main(String [] args) {

        // Obtain class-object for the object to be created in this JVM:

        Class clazz = null;

        try {
            clazz = Class.forName(args [0]);
        } catch (ClassNotFoundException e) {
            System.err.println("Could not find class: " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (IllegalArgumentException e) {
            System.err.println("Syntax in class name: " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
 
        // Locate registry in parent process:

        Registry registry = null ;

        try {
            registry = LocateRegistry.getRegistry(Integer.parseInt(args [1]));
        } catch (NumberFormatException e) {
            System.err.println("Illegal port number: " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (RemoteException e) {
            System.err.println("Failed to locate registry: " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }

        // ... hence look up parent call-back channel:

        RemoteChannel callBack = null;

        try {
            callBack = (RemoteChannel) registry.lookup("RemoteChannel");
        } catch (NotBoundException e) {
            System.err.println("No call-back channel in registry?: "
                               + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (RemoteException e) {
            System.err.println("Error in registry lookup: " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }

        // Create and return a slave object to the parent process:

        try {
            callBack.put((Remote) clazz.newInstance());
        } catch (InstantiationException e) {
            System.err.println("Class couldn't be instantiated: "
                               + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (IllegalAccessException e) {
            System.err.println("Access error creating instance: "
                               + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (RemoteException e) {
            System.err.println("Failed returning slave object: "
                               + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }

    public Slave() {}
}
